import {CustomContainer, CustomType} from './models/custom-container';

import Diagram = MindFusion.Diagramming.Diagram;
import HandlesStyle = MindFusion.Diagramming.HandlesStyle;
import Rect = MindFusion.Drawing.Rect;
import GlassEffect = MindFusion.Diagramming.GlassEffect;
import shape =MindFusion.Diagramming.Shape;
import Font = MindFusion.Drawing.Font;
import Events = MindFusion.Diagramming.Events;
import CoantainerNode = MindFusion.Diagramming.ContainerNode;
import TableNode = MindFusion.Diagramming.TableNode;
import Alignment = MindFusion.Diagramming.Alignment;
import Text = MindFusion.Drawing.Text;

export class CustomConatinerHelper {
    _ctr: any;
    private _factory: any;
    private _i: number = 0;
    private _height: number;
    private _conatinerAtt : CustomContainer
    constructor(factory : any, conatinerAtt: CustomContainer, height){
        this._factory = factory;
        this._height = height;
        this._conatinerAtt = conatinerAtt;
        this.createCustomTableTableNode();
    }
    createContainer(x: number, y: number) {
        if (this._conatinerAtt.type === CustomType.containerNode) {
            this._ctr = this._factory.createContainerNode(x, y, this._conatinerAtt.width, this._conatinerAtt.height);
            this._ctr.setCaptionHeight(this._conatinerAtt.height);
            //this._ctr.setFolded(true);
            var r = this._ctr.getBounds();
            r.height = this._height;
            this._ctr.setBounds(r);
            
        } else {
            this._ctr = this._factory.createShapeNode(x, y, this._conatinerAtt.width, this._conatinerAtt.height);
            this.addBar(x, y);
        }
        this._ctr.setShadowColor("transparent");
        this._ctr.setPen(this._conatinerAtt.headingColor);
        this.addHeaderLabel(x, y);
        this.addImage(x, y);
        this.addHeaderSupportLabel(x, y);
    }
    addNode(x, y, columnText, columnTextColor, columnImageSrcUrl) {
        let row = this._factory.createShapeNode(x, y + this._conatinerAtt.height + this._i, this._conatinerAtt.width, 10);
        let image = this._factory.createShapeNode(x + 4, y + this._conatinerAtt.height + 2 + this._i, 5, 5);
        image.setBrush(this._conatinerAtt.brushColor);
        image.setPen(this._conatinerAtt.penColor);
        //image.setShape(shape.fromId("Circle"));
        image.setImageLocation(columnImageSrcUrl);
        image.setLocked(true);
        this._ctr.add(image); 
        //this.attachNode(image);
        row.setText(columnText);
        row.setBrush("White");
        this._i += 10;
        row.setTextColor(columnTextColor);
        
        this._ctr.add(row); 
    }
    addFooter(x, y, incoming, outgoing) {
        this.inComingLinkText(x, y, incoming);
        this.outGoingLinkText(x, y, outgoing);
    }

    private attachNode(node){
        node.attachTo(this._ctr);
    }

    private addBar(x, y) {
        var bar = this._factory.createShapeNode(x + 5, y + 2, 2, 35);
        bar.setBrush(this._conatinerAtt.headingColor);
        bar.setPen(this._conatinerAtt.headingColor);
        bar.setLocked(true);
        this.attachNode(bar);
    }

    private addHeaderLabel(x, y){
        let l1 = this._factory.createShapeNode(x + 30, y - 8, this._conatinerAtt.height, this._conatinerAtt.height);
        l1.setFont(new Font(this._conatinerAtt.fontName, this._conatinerAtt.fontSize, false, false, false));
        l1.setTextColor(this._conatinerAtt.headingColor);
        l1.setText(this._conatinerAtt.headingText);
        l1.setTransparent(true);
        l1.setLocked(true);
        this.attachNode(l1);
    }

    private addHeaderSupportLabel(x, y){
        let l1 = this._factory.createShapeNode(x + 22, y, this._conatinerAtt.height, this._conatinerAtt.height);
        l1.setFont(new Font(this._conatinerAtt.fontName, this._conatinerAtt.fontSize, false, false, false));
        l1.setTextColor(this._conatinerAtt.supportTextcolor);
        l1.setText(this._conatinerAtt.headingSupportText);
        l1.setTransparent(true);
        l1.setLocked(true);
        this.attachNode(l1);
    }

    private addImage(x,y){
        //var image = this._factory.createShapeNode(x+10, y+12, 10, 10);
        //image.setBrush(this._conatinerAtt.brushColor);
        //image.setPen(this._conatinerAtt.penColor);
        //if(this._conatinerAtt.isCircularImage)
        //    image.setShape(shape.fromId("Circle"));
        //image.setImageLocation(this._conatinerAtt.imageUrl);
        //image.setLocked(true);
        //this.attachNode(image);
    }

    private createCustomTableTableNode() {
        
        MindFusion.Diagramming.ContainerNode.prototype.onUpdateVisuals = (item: MindFusion.Diagramming.DiagramItem) => {
            var rect = item.bounds;
            //rect.height = 40;
            rect.width = this._conatinerAtt.width
            item.setBounds(rect);
            item.setFoldable(true);
            item.setFoldIconSize(10);
            var rectPath = new MindFusion.Drawing.Path();
            let tempWidth = this._conatinerAtt.height - 4;
            rectPath.addRect(rect.x + 2, rect.y + 2, 4, tempWidth);
            rectPath.setPen(this._conatinerAtt.headingColor);
            rectPath.setBrush(this._conatinerAtt.headingColor);
            item.getGraphicsContent().push(rectPath);

            //var nodeText = new MindFusion.Drawing.Text(this._conatinerAtt.headingText, new MindFusion.Drawing.Rect(rect.x + 30, rect.y + 2, rect.width / 2, rect.width/2));
            //nodeText.setFont(new MindFusion.Drawing.Font("sans-serif", 4, false, true, false));
            //item.getGraphicsContent().push(nodeText);

            var image = new MindFusion.Drawing.Image(new Rect(rect.x + 10, rect.y + 5, 15, 15));
            image.image.src = this._conatinerAtt.imageUrl;
            image.image.width = 10;
            image.image.height = 10;
            image.loaded = true;
            item.getGraphicsContent().push(image); 

            image = new MindFusion.Drawing.Image(new Rect(rect.x + 20, rect.y + 20, 8, 8));
            image.image.src = this._conatinerAtt.incomingLnkImg;
            image.image.width = 10;
            image.image.height = 10;
            image.loaded = true;
            item.getGraphicsContent().push(image); 

            image = new MindFusion.Drawing.Image(new Rect(rect.x + 35, rect.y + 20, 8, 8));
            image.image.src = this._conatinerAtt.outgoingLnkImg;
            image.image.width = 10;
            image.image.height = 10;
            image.loaded = true;
            item.getGraphicsContent().push(image); 
        };
    }

    private inComingLinkText(x, y, incoming) {
        let outgoingText = this._factory.createShapeNode(x + this._conatinerAtt.height - 10 + 8, y + this._conatinerAtt.height - 10, 8, 8);
        outgoingText.setBrush("White");
        outgoingText.setPen("White");
        outgoingText.setShadowColor("transparent");
        outgoingText.setText(incoming);
        outgoingText.setLocked(true);
        this.attachNode(outgoingText);
    }

    private outGoingLinkText(x, y, outgoing) {
        let incomingText = this._factory.createShapeNode(x + this._conatinerAtt.height + 6 + 8, y + this._conatinerAtt.height - 10, 8, 8);
        incomingText.setText(outgoing);
        incomingText.setBrush("White");
        incomingText.setPen("White");
        incomingText.setShadowColor("transparent");
        incomingText.setLocked(true);
        this.attachNode(incomingText);
    }
}